﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using UnityEngine.UI;


namespace Mod
{
    public enum healthStates
    {
        dead,
        small,
        tall,
        fire,
        star,
        standard,
        squashed
    }

    public class Mod : MonoBehaviour
    {
        public static event System.Action OnChanged;
        public static void OnLoad()
        {
            ModAPI.RegisterInput("Mario's Jump", "mario jump", KeyCode.W);
            ModAPI.RegisterInput("Shoot Mario Fireball", "mario fire", KeyCode.F);

            ModAPI.RegisterInput("Luigi's Jump", "luigi jump", KeyCode.I);
            ModAPI.RegisterInput("Luigi Left", "luigi left", KeyCode.J);
            ModAPI.RegisterInput("Luigi Right", "luigi right", KeyCode.L);
            ModAPI.RegisterInput("Luigi Sprint", "luigi sprint", KeyCode.I);
            ModAPI.RegisterInput("Shoot Luigi Fireball", "luigi fire", KeyCode.O);
        }
        public static void Main()
        {
            ModAPI.Notify("A for left, D for right, W for jump, F to shoot fireballs, E or Q to place blocks, and left click to deselect current block");

            ModAPI.RegisterCategory("Mario Category", "All the mario stuff", ModAPI.LoadSprite("Textures/powerBox.png"));
            ModAPI.RegisterCategory("Blocks Category", "All the mario blocks", ModAPI.LoadSprite("Textures/BlocksTab.png"));
            ModAPI.RegisterCategory("Power Up Category", "All the mario Power Ups", ModAPI.LoadSprite("Textures/PowerTab.png"));

            #region Entities
            //Mario
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Mario",
                    DescriptionOverride = "Mario, from mario",
                    CategoryOverride = ModAPI.FindCategory("Mario Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/MarioIcon.png"),

                    AfterSpawn = (Instance) =>
                    {

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/mario.png");
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");

                        Instance.AddComponent<CapsuleCollider2D>();
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());

                        Instance.AddComponent<Mario>();

                        Instance.GetComponent<Mario>().mario = Instance.gameObject;
                        Instance.GetComponent<Mario>().jumpSoundSmall = ModAPI.LoadSound("Textures/Sounds/jumpSound.mp3");
                        Instance.GetComponent<Mario>().jumpSoundAdult = ModAPI.LoadSound("Textures/Sounds/smb_jump-super.wav");
                        Instance.GetComponent<Mario>().fireballSound = ModAPI.LoadSound("Textures/Sounds/smb_fireball.wav");
                        Instance.GetComponent<Mario>().state = healthStates.small;
                        Instance.GetComponent<Mario>().powerUpSound = ModAPI.LoadSound("Textures/Sounds/smb_powerup.wav");
                        Instance.GetComponent<Mario>().rb = Instance.GetComponent<Rigidbody2D>();
                        Instance.GetComponent<Mario>().col = Instance.GetComponent<CapsuleCollider2D>();
                        Instance.GetComponent<Mario>().sr = Instance.GetComponent<SpriteRenderer>();
                        Instance.GetComponent<Mario>().jumpForce = 170;
                        Instance.GetComponent<Mario>().jumpFalloff = 2;

                        Instance.transform.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .45f);
                        Instance.transform.localScale = new Vector3(2, 2, 2);

                        Instance.GetComponent<SpriteRenderer>().sortingOrder = 1;

                        #region Sprites

                        #region small
                        Instance.GetComponent<Mario>().smallIdle = ModAPI.LoadSprite("Textures/mario.png");
                        Instance.GetComponent<Mario>().smallRun1 = ModAPI.LoadSprite("Textures/RunAnim/run1.png");
                        Instance.GetComponent<Mario>().smallRun2 = ModAPI.LoadSprite("Textures/RunAnim/run2.png");
                        Instance.GetComponent<Mario>().smallRun3 = ModAPI.LoadSprite("Textures/RunAnim/run3.png");
                        Instance.GetComponent<Mario>().smallJump = ModAPI.LoadSprite("Textures/RunAnim/jump.png");
                        Instance.GetComponent<Mario>().smallDeath = ModAPI.LoadSprite("Textures/marioDeath.png");
                        #endregion

                        #region cur
                        Instance.GetComponent<Mario>().curIdle = ModAPI.LoadSprite("Textures/mario.png");
                        Instance.GetComponent<Mario>().curRun1 = ModAPI.LoadSprite("Textures/RunAnim/run1.png");
                        Instance.GetComponent<Mario>().curRun2 = ModAPI.LoadSprite("Textures/RunAnim/run2.png");
                        Instance.GetComponent<Mario>().curRun3 = ModAPI.LoadSprite("Textures/RunAnim/run3.png");
                        Instance.GetComponent<Mario>().curJump = ModAPI.LoadSprite("Textures/RunAnim/jump.png");
                        Instance.GetComponent<Mario>().smallDeath = ModAPI.LoadSprite("Textures/marioDeath.png");
                        #endregion

                        #region adult
                        Instance.GetComponent<Mario>().adultIdle = ModAPI.LoadSprite("Textures/RunAnim/GrownUp/AdultIdle.png");
                        Instance.GetComponent<Mario>().adultRun1 = ModAPI.LoadSprite("Textures/RunAnim/GrownUp/AdultRun1.png");
                        Instance.GetComponent<Mario>().adultRun2 = ModAPI.LoadSprite("Textures/RunAnim/GrownUp/AdultRun2.png");
                        Instance.GetComponent<Mario>().adultRun3 = ModAPI.LoadSprite("Textures/RunAnim/GrownUp/AdultRun3.png");
                        Instance.GetComponent<Mario>().adultJump = ModAPI.LoadSprite("Textures/RunAnim/GrownUp/AdultJump.png");
                        #endregion

                        #region fire
                        Instance.GetComponent<Mario>().fireIdle = ModAPI.LoadSprite("Textures/RunAnim/Fire/FireIdle.png");
                        Instance.GetComponent<Mario>().fireRun1 = ModAPI.LoadSprite("Textures/RunAnim/Fire/FireRun1.png");
                        Instance.GetComponent<Mario>().fireRun2 = ModAPI.LoadSprite("Textures/RunAnim/Fire/FireRun2.png");
                        Instance.GetComponent<Mario>().fireRun3 = ModAPI.LoadSprite("Textures/RunAnim/Fire/FireRun3.png");
                        Instance.GetComponent<Mario>().fireJump = ModAPI.LoadSprite("Textures/RunAnim/Fire/FireJump.png");
                        Instance.GetComponent<Mario>().fireShoot = ModAPI.LoadSprite("Textures/RunAnim/Fire/shootingFire.png");
                        #endregion

                        #endregion
                    }

                });

            //Luigi
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Luigi",
                    DescriptionOverride = "Luigi, from mario",
                    CategoryOverride = ModAPI.FindCategory("Mario Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/LuigiIcon.png"),

                    AfterSpawn = (Instance) =>
                    {

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/mario.png");
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");

                        Instance.AddComponent<CapsuleCollider2D>();
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());

                        Instance.AddComponent<Luigi>();

                        Instance.GetComponent<Luigi>().luigi = Instance.gameObject;
                        Instance.GetComponent<Luigi>().jumpSoundSmall = ModAPI.LoadSound("Textures/Sounds/jumpSound.mp3");
                        Instance.GetComponent<Luigi>().jumpSoundAdult = ModAPI.LoadSound("Textures/Sounds/smb_jump-super.wav");
                        Instance.GetComponent<Luigi>().fireballSound = ModAPI.LoadSound("Textures/Sounds/smb_fireball.wav");
                        Instance.GetComponent<Luigi>().state = healthStates.small;
                        Instance.GetComponent<Luigi>().powerUpSound = ModAPI.LoadSound("Textures/Sounds/smb_powerup.wav");
                        Instance.GetComponent<Luigi>().rb = Instance.GetComponent<Rigidbody2D>();
                        Instance.GetComponent<Luigi>().col = Instance.GetComponent<CapsuleCollider2D>();
                        Instance.GetComponent<Luigi>().sr = Instance.GetComponent<SpriteRenderer>();
                        Instance.GetComponent<Luigi>().jumpForce = 180;
                        Instance.GetComponent<Luigi>().jumpFalloff = 1.8f;

                        Instance.transform.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .45f);
                        Instance.transform.localScale = new Vector3(2, 2, 2);

                        Instance.GetComponent<SpriteRenderer>().sortingOrder = 1;

                        #region Sprites

                        #region small
                        Instance.GetComponent<Luigi>().smallIdle = ModAPI.LoadSprite("Textures/RunAnim/Luigi/idle.png");
                        Instance.GetComponent<Luigi>().smallRun1 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/run1.png");
                        Instance.GetComponent<Luigi>().smallRun2 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/run2.png");
                        Instance.GetComponent<Luigi>().smallRun3 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/run3.png");
                        Instance.GetComponent<Luigi>().smallJump = ModAPI.LoadSprite("Textures/RunAnim/Luigi/jump.png");
                        Instance.GetComponent<Luigi>().smallDeath = ModAPI.LoadSprite("Textures/Luigi/death.png");
                        #endregion

                        #region cur
                        Instance.GetComponent<Luigi>().curIdle = ModAPI.LoadSprite("Textures/mario.png");
                        Instance.GetComponent<Luigi>().curRun1 = ModAPI.LoadSprite("Textures/RunAnim/run1.png");
                        Instance.GetComponent<Luigi>().curRun2 = ModAPI.LoadSprite("Textures/RunAnim/run2.png");
                        Instance.GetComponent<Luigi>().curRun3 = ModAPI.LoadSprite("Textures/RunAnim/run3.png");
                        Instance.GetComponent<Luigi>().curJump = ModAPI.LoadSprite("Textures/RunAnim/jump.png");
                        Instance.GetComponent<Luigi>().smallDeath = ModAPI.LoadSprite("Textures/marioDeath.png");
                        #endregion

                        #region adult
                        Instance.GetComponent<Luigi>().adultIdle = ModAPI.LoadSprite("Textures/RunAnim/Luigi/luigiAIdle.png");
                        Instance.GetComponent<Luigi>().adultRun1 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/luigiA1.png");
                        Instance.GetComponent<Luigi>().adultRun2 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/luigiA2.png");
                        Instance.GetComponent<Luigi>().adultRun3 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/luigiA3.png");
                        Instance.GetComponent<Luigi>().adultJump = ModAPI.LoadSprite("Textures/RunAnim/Luigi/luigiAJump.png");
                        #endregion

                        #region fire
                        Instance.GetComponent<Luigi>().fireIdle = ModAPI.LoadSprite("Textures/RunAnim/Luigi/fireI.png");
                        Instance.GetComponent<Luigi>().fireRun1 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/fire1.png");
                        Instance.GetComponent<Luigi>().fireRun2 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/fire2.png");
                        Instance.GetComponent<Luigi>().fireRun3 = ModAPI.LoadSprite("Textures/RunAnim/Luigi/fire3.png");
                        Instance.GetComponent<Luigi>().fireJump = ModAPI.LoadSprite("Textures/RunAnim/Luigi/fireJ.png");
                        Instance.GetComponent<Luigi>().fireShoot = ModAPI.LoadSprite("Textures/RunAnim/Luigi/fireS.png");
                        #endregion

                        #endregion
                    }

                });

            //Goomba
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Goomba",
                    DescriptionOverride = "Mario's most iconic enemy, i think.",
                    CategoryOverride = ModAPI.FindCategory("Mario Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Enemies/Goomba/GoombaPrev.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");

                        Instance.AddComponent<CapsuleCollider2D>();
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());

                        Instance.transform.GetComponent<CapsuleCollider2D>().size = new Vector2(0.43f, .43f);
                        Instance.transform.localScale = new Vector3(2, 2, 2);
                        Instance.name = "enemy";

                        Instance.AddComponent<Goomba>();

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Enemies/Goomba/Goomba1.png");

                        Instance.GetComponent<Goomba>().enemy = Instance.transform;
                        Instance.GetComponent<Goomba>().rb = Instance.GetComponent<Rigidbody2D>();

                        Instance.GetComponent<Goomba>().run1 = ModAPI.LoadSprite("Textures/Enemies/Goomba/Goomba1.png");
                        Instance.GetComponent<Goomba>().run2 = ModAPI.LoadSprite("Textures/Enemies/Goomba/Goomba2.png");
                        Instance.GetComponent<Goomba>().squashed = ModAPI.LoadSprite("Textures/Enemies/Goomba/Goomba3.png");

                        Instance.GetComponent<Goomba>().squashSound = ModAPI.LoadSound("Textures/Sounds/squash.mp3");
                    }
                });

            //Koopa
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Koopa",
                    DescriptionOverride = "Mario's second most iconic enemy, i think.",
                    CategoryOverride = ModAPI.FindCategory("Mario Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Enemies/Koopa/KoopaPrev.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");

                        Instance.AddComponent<CapsuleCollider2D>();
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());

                        Instance.transform.GetComponent<CapsuleCollider2D>().size = new Vector2(0.43f, .66f);
                        Instance.transform.localScale = new Vector3(2, 2, 2);
                        Instance.name = "enemy";

                        Instance.AddComponent<Koopa>();
                        Instance.AddComponent<Koopa>().state = healthStates.standard;

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Enemies/Koopa/Koopa1.png");

                        Instance.GetComponent<Koopa>().enemy = Instance.transform;
                        Instance.GetComponent<Koopa>().rb = Instance.GetComponent<Rigidbody2D>();

                        Instance.GetComponent<Koopa>().Hurt(0.01f);

                        Instance.GetComponent<Koopa>().run1 = ModAPI.LoadSprite("Textures/Enemies/Koopa/Koopa1.png");
                        Instance.GetComponent<Koopa>().run2 = ModAPI.LoadSprite("Textures/Enemies/Koopa/Koopa2.png");
                        Instance.GetComponent<Koopa>().squashed = ModAPI.LoadSprite("Textures/Enemies/Koopa/Koopa3.png");


                    }
                });

            //rKoopa
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Red Koopa",
                    DescriptionOverride = "Mario's second most iconic reskin, i think.",
                    CategoryOverride = ModAPI.FindCategory("Mario Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Enemies/rKoopa/rKoopaPrev.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");

                        Instance.AddComponent<CapsuleCollider2D>();
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());

                        Instance.transform.GetComponent<CapsuleCollider2D>().size = new Vector2(0.43f, .66f);
                        Instance.transform.localScale = new Vector3(2, 2, 2);
                        Instance.name = "enemy";

                        Instance.AddComponent<Koopa>();
                        Instance.AddComponent<Koopa>().state = healthStates.standard;

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Enemies/rKoopa/rKoopa1.png");

                        Instance.GetComponent<Koopa>().enemy = Instance.transform;
                        Instance.GetComponent<Koopa>().rb = Instance.GetComponent<Rigidbody2D>();
                        Instance.GetComponent<Koopa>().red = true;

                        Instance.GetComponent<Koopa>().Hurt(0.01f);

                        Instance.GetComponent<Koopa>().run1 = ModAPI.LoadSprite("Textures/Enemies/rKoopa/rKoopa1.png");
                        Instance.GetComponent<Koopa>().run2 = ModAPI.LoadSprite("Textures/Enemies/rKoopa/rKoopa2.png");
                        Instance.GetComponent<Koopa>().squashed = ModAPI.LoadSprite("Textures/Enemies/rKoopa/rKoopa3.png");


                    }
                });

            #endregion

            #region Blocks

            //Brick tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "OW Brick Block",
                    DescriptionOverride = "Building block for all mario levels",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Brick.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Brick");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Brick.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        Instance.GetComponent<StandardBlock>().breakSound = ModAPI.LoadSound("Textures/Sounds/smb_breakblock.wav");
                        Instance.GetComponent<StandardBlock>().canBeBroken = true;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //Brick tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "UW Brick Block",
                    DescriptionOverride = "Building block for all mario levels",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/UWBrick.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Brick");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/UWBrick.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        Instance.GetComponent<StandardBlock>().breakSound = ModAPI.LoadSound("Textures/Sounds/smb_breakblock.wav");
                        Instance.GetComponent<StandardBlock>().canBeBroken = true;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //floor tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "OW Floor Block",
                    DescriptionOverride = "Idk what this block is supposed to be but it is used as the standard floor block... for some reason.",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Floor.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Floor.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //floor tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "UW Floor Block",
                    DescriptionOverride = "Flooring from underground",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/UWFloor.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/UWFloor.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //metal tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "OW Metal Block",
                    DescriptionOverride = "Block of metal",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Metal.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Metal.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //Uw metal tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "UW Metal Block",
                    DescriptionOverride = "Block of metal from underground",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/UWMetal.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/UWMetal.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //Ow Rock tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "OW Mountain Rocks",
                    DescriptionOverride = "Rocks from a mountain",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Rock.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Rock.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //UW Rock tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "UW Mountain Rocks",
                    DescriptionOverride = "Rocks from an underground mountain",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/UWRock.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/UWRock.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //Castle Floors tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "castle Floors",
                    DescriptionOverride = "Floors in a castle",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/castleFloor.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/castleFloor.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //Metal Castle tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "castle Metal",
                    DescriptionOverride = "Metal in a castle",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/castleMetal.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/castleMetal.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //empty power up block
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Empty Block",
                    DescriptionOverride = "Empty power up block",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/emptyBlock.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Hollow Metal");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/emptyBlock.png");

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = true;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //Castle brick tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "castle Brick",
                    DescriptionOverride = "Bricks in a castle",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/castleBrick.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rock");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/castleBrick.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        Instance.GetComponent<StandardBlock>().canBeBroken = true;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            #endregion

            #region Background Tiles
            //Bush tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Bush",
                    DescriptionOverride = "totally not a reskinned cloud...",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Bush.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Brick");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.GetComponent<BoxCollider2D>().isTrigger = true;
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Bush.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;


                        Instance.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());
                    }
                });

            //Cloud tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Cloud",
                    DescriptionOverride = "totally not a reskinned bush...",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Cloud.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Brick");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.GetComponent<BoxCollider2D>().isTrigger = true;
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Cloud.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;

                        Instance.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());
                    }
                });

            //Castle tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Castle",
                    DescriptionOverride = "The castle mario exits to at the end of every level.",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/Castle.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Brick");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.GetComponent<BoxCollider2D>().isTrigger = true;
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/Castle.png");
                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;

                        Instance.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());

                    }
                });

            //OW Background Tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "OW Background",
                    DescriptionOverride = "The standard background for overworld levels",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/OWbackground.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/OWbackground.png");
                        Instance.GetComponent<SpriteRenderer>().sortingOrder = -20;

                        Instance.AddComponent<SquareDraw>();
                        Instance.AddComponent<SquareDraw>().square = Instance.gameObject;
                        Instance.GetComponent<SquareDraw>().backgroundSprite = ModAPI.LoadSprite("Textures/Blocks/OWbackground.png");

                        ModAPI.Notify("Drag click to place");

                        Instance.transform.localScale = new Vector3(0, 0, 2);

                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());
                    }
                });

            //UW Background Tile
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "UW Background",
                    DescriptionOverride = "The standard background for underworld levels",
                    CategoryOverride = ModAPI.FindCategory("Blocks Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/UWPrev.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/UWbackground.png");
                        Instance.GetComponent<SpriteRenderer>().sortingOrder = -20;

                        Instance.AddComponent<SquareDraw>();
                        Instance.AddComponent<SquareDraw>().square = Instance.gameObject;
                        Instance.GetComponent<SquareDraw>().backgroundSprite = ModAPI.LoadSprite("Textures/Blocks/UWbackground.png");

                        ModAPI.Notify("Drag click to place");

                        Instance.transform.localScale = new Vector3(0, 0, 2);

                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());
                    }
                });

            #endregion

            #region Items

            //mushroom
            ModAPI.Register(new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                NameOverride = "Mushroom",
                DescriptionOverride = "I wonder what happens to you when you eat this..",
                CategoryOverride = ModAPI.FindCategory("Power Up Category"),
                ThumbnailOverride = ModAPI.LoadSprite("Textures/Items/mushroom.png"),

                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Items/mushroom.png");
                    Instance.AddComponent<Mushroom>();
                    Instance.AddComponent<Mushroom>().rb = Instance.GetComponent<Rigidbody2D>();
                    Instance.AddComponent<Mushroom>().moveDir = 1;

                    Instance.GetComponent<Rigidbody2D>().freezeRotation = true;

                    Instance.GetComponent<BoxCollider2D>().size = new Vector2(0.43f, 0.43f);
                    Instance.transform.localScale = new Vector3(2, 2, 2);
                }
            });

            //fire flower
            ModAPI.Register(new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                NameOverride = "Fire Flower",
                DescriptionOverride = "Spicy",
                CategoryOverride = ModAPI.FindCategory("Power Up Category"),
                ThumbnailOverride = ModAPI.LoadSprite("Textures/Items/FireFlower.png"),

                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Items/FireFlower.png");
                    Instance.AddComponent<FireFlower>();

                    Instance.GetComponent<Rigidbody2D>().freezeRotation = true;

                    Instance.GetComponent<BoxCollider2D>().size = new Vector2(0.43f, 0.43f);
                    Instance.transform.localScale = new Vector3(2, 2, 2);
                }
            });

            //power up Fire Flower
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Power Up (Holding a Mushroom)",
                    DescriptionOverride = "Power Up block",
                    CategoryOverride = ModAPI.FindCategory("Power Up Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/PowerUpMushroom.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Hollow Metal");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/powerUp.png");

                        Instance.GetComponent<StandardBlock>().canHoldItems = true;

                        Instance.GetComponent<StandardBlock>().itemName = "Mushroom";
                        Instance.GetComponent<StandardBlock>().animated = true;
                        Instance.GetComponent<StandardBlock>().block1 = ModAPI.LoadSprite("Textures/Blocks/powerUp.png");
                        Instance.GetComponent<StandardBlock>().block2 = ModAPI.LoadSprite("Textures/Blocks/powerUp1.png");
                        Instance.GetComponent<StandardBlock>().block3 = ModAPI.LoadSprite("Textures/Blocks/powerUp2.png");

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });

            //power up block
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Power Up (Holding A Fire Flower)",
                    DescriptionOverride = "Power Up block",
                    CategoryOverride = ModAPI.FindCategory("Power Up Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Blocks/PowerUpFire.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.transform.position = Global.main.MousePosition;
                        Instance.AddComponent<StandardBlock>();
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Hollow Metal");

                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, 0.45f);
                        Instance.transform.localScale = new Vector3(2.2f, 2.2f, 2);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/powerUp.png");

                        Instance.GetComponent<StandardBlock>().canHoldItems = true;

                        Instance.GetComponent<StandardBlock>().itemName = "Fire Flower";
                        Instance.GetComponent<StandardBlock>().animated = true;
                        Instance.GetComponent<StandardBlock>().block1 = ModAPI.LoadSprite("Textures/Blocks/powerUp.png");
                        Instance.GetComponent<StandardBlock>().block2 = ModAPI.LoadSprite("Textures/Blocks/powerUp1.png");
                        Instance.GetComponent<StandardBlock>().block3 = ModAPI.LoadSprite("Textures/Blocks/powerUp2.png");

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<BoxCollider2D>().enabled = false;

                        Instance.GetComponent<StandardBlock>().block = Instance.gameObject;
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                    }
                });
            #endregion

            #region bosses
            /*
            //Bowser
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Bowser",
                    DescriptionOverride = "Bowser, he needs no introduction",
                    CategoryOverride = ModAPI.FindCategory("Mario Category"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Enemies/Bowser.png"),

                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Instance.transform.GetComponent<BoxCollider2D>().size = new Vector2(0.5f, .9f);
                        Instance.transform.localScale = new Vector3(2, 2, 2);
                        Instance.name = "enemy";

                        Instance.AddComponent<Bowser>();

                        Instance.GetComponent<Rigidbody2D>().freezeRotation = true;
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Enemies/Bowser.png");


                    }
                });
            */
            #endregion

            //fireball
            ModAPI.Register(new Modification()
            {
                NameOverride = "Fire Ball",
                DescriptionOverride = "The most iconic fireball... not really a cool title.",
                OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                CategoryOverride = ModAPI.FindCategory("Mario Category"),
                ThumbnailOverride = ModAPI.LoadSprite("Textures/Fire/fireBall.png"),
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Fire/fireBall.png");
                    Instance.AddComponent<FireBall>();

                    UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");

                    Instance.AddComponent<CircleCollider2D>();
                    Instance.transform.localScale = new Vector3(2, 2, 2);
                    Instance.GetComponent<FireBall>().col = Instance.GetComponent<CircleCollider2D>();
                    Instance.GetComponent<FireBall>().rb = Instance.GetComponent<Rigidbody2D>();
                }
            });
        }
    }

    //mario controller

    public abstract class Character : MonoBehaviour 
    {
        public healthStates state;

        public Sprite curSprite;

        public Sprite curIdle, curRun1, curRun2, curRun3, curJump;

        public Sprite smallIdle, smallRun1, smallRun2, smallRun3, smallJump, smallDeath;

        public Sprite adultIdle, adultRun1, adultRun2, adultRun3, adultJump;

        public Sprite fireIdle, fireRun1, fireRun2, fireRun3, fireJump, fireShoot;

        public float jumpForce;
        public float jumpFalloff;

        public abstract void HandleJumping();
        public abstract void HandleBreaking();
        public abstract void HandleInput();
        public abstract void HandleAnim();
        public abstract void HandleSprites();
        public abstract void Jump(float jumpForce, bool fallFast, bool playSound);
        public abstract void Hurt();
        public abstract void Die();
        public abstract void PowerUp(healthStates powerType);
        public abstract void ShootFire();
    }


    public class Mario : Character
    {
        #region variables

        public static Mario Instance;

        public AudioClip jumpSoundSmall;
        public AudioClip jumpSoundAdult;

        public AudioClip powerUpSound;
        public AudioClip fireballSound;

        public GameObject mario;

        public Rigidbody2D rb;

        public SpriteRenderer sr;

        public CapsuleCollider2D col;

        public bool controllable = true;
        public bool adult = false;

        private float runAnimTime;
        private float jumpRange = .53f;

        float cooldown = 1;
        float fireSpriteCooldown = 0;
        public float invincibilityFrames = 0;

        private bool moving;

        int animNumb;

        public float maxSpeed = 7;
        public float sprintSpeed = 12;

        public float moveSpeed = 500;

        bool canJump;
        #endregion

        private void FixedUpdate()
        {
            rb.mass = 16;
            rb.useAutoMass = false;

            Vector2 moveDir = new Vector2(Input.GetAxisRaw("Horizontal"), 0);

            rb.AddForce(mario.transform.right * moveDir * moveSpeed);

            rb.velocity = new Vector2(Mathf.Clamp(rb.velocity.x, -maxSpeed, maxSpeed), rb.velocity.y);

            HandleBreaking();
        }

        public void Update()
        {
            Instance = this;

            if (!controllable) { return; }

            sr.sprite = curSprite;

            invincibilityFrames -= Time.deltaTime;
            fireSpriteCooldown -= Time.deltaTime;

            if (state == healthStates.dead)
            {
                curSprite = smallDeath;
                Die();
            }

            if (state == healthStates.fire)
            {
                if (InputSystem.Down("mario fire"))
                {
                    ShootFire();
                }
            }

            if (state != healthStates.dead)
            {
                HandleInput();

                if (moving && fireSpriteCooldown <= 0)
                    HandleAnim();
                else if (fireSpriteCooldown > 0)
                {
                    curSprite = fireShoot;
                }

                HandleJumping();

                //if off the ground, set sprite to jumping sprite
                if (!canJump && state != healthStates.dead) { curSprite = curJump; }
            }

            PhysicsMaterial2D mat = new PhysicsMaterial2D();

            mat.friction = 1f;
            mat.bounciness = 0;

            rb.sharedMaterial = mat;

            HandleSprites();
        }

        #region Handlers

        public override void HandleJumping()
        {
            //tell if on the ground
            RaycastHit2D hit = Physics2D.Raycast(transform.position, Vector2.down, jumpRange);
            if (hit)
            {
                canJump = true;
                rb.gravityScale = 1;
            }
            else canJump = false;

            if (canJump && Input.GetKeyDown(KeyCode.W)) Jump(jumpForce, false, true);

            if (hit.transform.name == "enemy")
            {
                hit.transform.GetComponent<BasicEnemy>().Hurt(5);
                Jump(jumpForce: 15, fallFast: true, playSound: false);

                if (hit.transform.GetComponent<Koopa>().squashed && hit.transform.GetComponent<Koopa>().shellTime <= 4.75f)
                {
                    hit.transform.GetComponent<Koopa>().hitInShell = true;
                }
            }
        }

        public override void HandleBreaking()
        {
            RaycastHit2D hit = Physics2D.Raycast(transform.position, transform.up, jumpRange);

            if (hit.transform.GetComponent<StandardBlock>())
            {
                hit.transform.GetComponent<StandardBlock>().Break();
            }
        }

        public override void HandleInput()
        {
            if (Input.GetAxisRaw("Horizontal") != 0)
            {
                moving = true;
            }
            else
            {
                moving = false;
            }

            if (Input.GetKeyUp(KeyCode.W)) { rb.gravityScale = jumpFalloff; }
            if (Input.GetKey(KeyCode.W)) { rb.gravityScale = 1; }

            //sprint
            if (Input.GetKey(KeyCode.LeftShift))
            {
                maxSpeed = sprintSpeed;
            }
            if (!Input.GetKey(KeyCode.LeftShift))
            {
                maxSpeed = 7;
            }

            if (!moving && canJump)
            {
                rb.drag = 15;
            }
            else
            {
                rb.drag = 0;
            }

            //if not moving, set sprite to idle
            if (!moving && canJump) curSprite = curIdle;
            else if(!canJump) { curSprite = curJump; }

            if (mario.GetComponent<PhysicalBehaviour>().IsInLava)
            {
                state = healthStates.dead;
            }
        }

        public override void HandleAnim()
        {
            if (Input.GetAxisRaw("Horizontal") < 0)
            {
                transform.localScale = new Vector3(-2, 2, 2);
            }
            if (Input.GetAxisRaw("Horizontal") > 0)
            {
                transform.localScale = new Vector3(2, 2, 2);
            }

            if (canJump)
            {
                //decrease timer
                runAnimTime -= Time.deltaTime;

                //reset number if too high
                if (animNumb > 3) animNumb = 1;

                //change keyframe number
                if (runAnimTime <= 0)
                {
                    runAnimTime = 0.7f / maxSpeed;
                    animNumb++;
                }

                //set sprite to current keyframe using int and array
                switch (animNumb)
                {
                    case 1:
                        curSprite = curRun1;
                        break;
                    case 2:
                        curSprite = curRun2;
                        break;
                    case 3:
                        curSprite = curRun3;
                        break;
                }
            }
        }

        public override void HandleSprites()
        {
            switch (state)
            {
                case healthStates.small:

                    curIdle = smallIdle;
                    curRun1 = smallRun1;
                    curRun2 = smallRun2;
                    curRun3 = smallRun3;
                    curJump = smallJump;

                    jumpRange = 0.53f;
                    mario.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .45f);

                    adult = false;

                    break;

                case healthStates.tall:

                    curIdle = adultIdle;
                    curRun1 = adultRun1;
                    curRun2 = adultRun2;
                    curRun3 = adultRun3;
                    curJump = adultJump;

                    jumpRange = 1f;
                    mario.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .93f);

                    adult = true;

                    break;

                case healthStates.fire:

                    curIdle = fireIdle;
                    curRun1 = fireRun1;
                    curRun2 = fireRun2;
                    curRun3 = fireRun3;
                    curJump = fireJump;

                    jumpRange = 1f;
                    mario.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .93f);

                    adult = true;

                    break;

            }
        }

        #endregion

        #region Controllers
        public override void Jump(float jumpForce, bool fallFast, bool playSound)
        {
            if (!canJump) { return; }

            if (fallFast) { rb.gravityScale = 2; }

            if (playSound && !adult)
            {
                GetComponent<PhysicalBehaviour>().PlayClipOnce(clip: jumpSoundSmall, volume: .5f);

            }
            else if (playSound && adult)
            {
                GetComponent<PhysicalBehaviour>().PlayClipOnce(clip: jumpSoundAdult, volume: 1);
            }

            //cant jump again, so you cant infinitely jump in the air
            canJump = false;

            //add force upwards for jump
            rb.AddForce(Vector2.up * jumpForce, ForceMode2D.Impulse);
        }

        public override void Hurt()
        {
            if (invincibilityFrames > 0) { return; }

            if (state == healthStates.small) { Die(); }
            if (state == healthStates.tall || state == healthStates.fire) { state = healthStates.small; HandleSprites(); invincibilityFrames = 1; }
        }

        public override void Die()
        {
            cooldown -= Time.fixedDeltaTime;

            curSprite = smallDeath;

            Jump(150, true, false);

            col.enabled = false;

            if (cooldown <= 0)
            {
                mario.SetActive(false);
            }
        }

        public override void PowerUp(healthStates powerType)
        {
            state = powerType;
            GetComponent<PhysicalBehaviour>().PlayClipOnce(powerUpSound, 0.5f);
        }

        public override void ShootFire()
        {
            fireSpriteCooldown = 0.15f;

            GameObject fireBall = Instantiate(ModAPI.FindSpawnable("Fire Ball").Prefab, transform.position + new Vector3(0.25f * (transform.localScale.x / transform.localScale.x), 0, 0), Quaternion.identity);
            CatalogBehaviour.PerformMod(ModAPI.FindSpawnable("Fire Ball"), fireBall);

            ModAPI.Notify("fireball shot");

            GetComponent<PhysicalBehaviour>().PlayClipOnce(fireballSound, 1f);

            fireBall.GetComponent<FireBall>().characterThatThrew = mario;
            fireBall.GetComponent<Rigidbody2D>().AddForce(fireBall.transform.right * transform.localScale.x * 75, ForceMode2D.Impulse);
        }

        #endregion
    }

    public class Luigi : Character
    {
        #region variables

        public static Luigi Instance;

        public AudioClip jumpSoundSmall;
        public AudioClip jumpSoundAdult;

        public AudioClip powerUpSound;
        public AudioClip fireballSound;

        public GameObject luigi;

        public Rigidbody2D rb;

        public SpriteRenderer sr;

        public CapsuleCollider2D col;

        public bool controllable = true;
        public bool adult = false;

        public healthStates state;

        private float runAnimTime;
        private float jumpRange = .53f;

        float xAxis;

        float cooldown = 1;
        float fireSpriteCooldown = 0;
        public float invincibilityFrames = 0;

        private bool moving;

        int animNumb;

        public float maxSpeed = 7;
        public float sprintSpeed = 12;

        public float moveSpeed = 500;

        bool canJump;
        #endregion

        private void FixedUpdate()
        {
            rb.mass = 16;
            rb.useAutoMass = false;

            Vector2 moveDir = new Vector2(xAxis, 0);

            rb.AddForce(luigi.transform.right * moveDir * moveSpeed);

            rb.velocity = new Vector2(Mathf.Clamp(rb.velocity.x, -maxSpeed, maxSpeed), rb.velocity.y);

            HandleBreaking();
        }

        public void Update()
        {

            Instance = this;
            if (!controllable) { return; }

            sr.sprite = curSprite;

            invincibilityFrames -= Time.deltaTime;
            fireSpriteCooldown -= Time.deltaTime;

            if (state == healthStates.dead)
            {
                curSprite = smallDeath;
                Die();
            }

            if (state == healthStates.fire)
            {
                if (InputSystem.Down("luigi fire"))
                {
                    ShootFire();
                }
            }

            if (state != healthStates.dead)
            {
                HandleInput();

                if (moving && fireSpriteCooldown <= 0)
                    HandleAnim();
                else if (fireSpriteCooldown > 0)
                {
                    curSprite = fireShoot;
                }

                HandleJumping();

                //if off the ground, set sprite to jumping sprite
                if (!canJump && state != healthStates.dead) { curSprite = curJump; }
            }

            PhysicsMaterial2D mat = new PhysicsMaterial2D();

            mat.friction = 1f;
            mat.bounciness = 0;

            rb.sharedMaterial = mat;

            HandleSprites();
        }

        #region Handlers

        public override void HandleJumping()
        {
            //tell if on the ground
            RaycastHit2D hit = Physics2D.Raycast(transform.position, Vector2.down, jumpRange);
            if (hit)
            {
                canJump = true;
                rb.gravityScale = 1;
            }
            else canJump = false;

            if (canJump && InputSystem.Down("luigi jump")) Jump(jumpForce, false, true);

            if (hit.transform.name == "enemy")
            {
                hit.transform.GetComponent<BasicEnemy>().Hurt(5);
                Jump(jumpForce: 15, fallFast: true, playSound: false);

                if (hit.transform.GetComponent<Koopa>().squashed && hit.transform.GetComponent<Koopa>().shellTime <= 4.75f)
                {
                    hit.transform.GetComponent<Koopa>().hitInShell = true;
                }
            }
        }

        public override void HandleBreaking()
        {
            RaycastHit2D hit = Physics2D.Raycast(transform.position, transform.up, jumpRange);

            if (hit.transform.GetComponent<StandardBlock>())
            {
                hit.transform.GetComponent<StandardBlock>().Break();
            }
        }

        public override void HandleInput()
        {
            if (InputSystem.Held("luigi left"))
            {
                xAxis = -1;
                transform.localScale = new Vector3(-2, 2, 2);
            }

            if (InputSystem.Held("luigi right"))
            {
                xAxis = 1;
                transform.localScale = new Vector3(2, 2, 2);
            }

            if(!InputSystem.Held("luigi left") && !InputSystem.Held("luigi right"))
            {
                xAxis = 0;
            }

            if (xAxis != 0)
            {
                moving = true;
            }
            else
            {
                moving = false;
            }

            if (InputSystem.Up("luigi jump")) { rb.gravityScale = jumpFalloff; }
            if (InputSystem.Held("luigi jump")) { rb.gravityScale = 0.9f; }

            //sprint
            if (InputSystem.Held("luigi sprint"))
            {
                maxSpeed = sprintSpeed;
            }
            if (!InputSystem.Held("luigi sprint"))
            {
                maxSpeed = 7;
            }

            if (!moving && canJump)
            {
                rb.drag = 15;
            }
            else
            {
                rb.drag = 0;
            }

            //if not moving, set sprite to idle
            if (!moving && canJump) curSprite = curIdle;
            else if(!canJump) { curSprite = curJump; }

            if (luigi.GetComponent<PhysicalBehaviour>().IsInLava)
            {
                state = healthStates.dead;
            }
        }

        public override void HandleAnim()
        {

            if (canJump)
            {
                //decrease timer
                runAnimTime -= Time.deltaTime;

                //reset number if too high
                if (animNumb > 3) animNumb = 1;

                //change keyframe number
                if (runAnimTime <= 0)
                {
                    runAnimTime = 0.7f / maxSpeed;
                    animNumb++;
                }

                //set sprite to current keyframe using int and array
                switch (animNumb)
                {
                    case 1:
                        curSprite = curRun1;
                        break;
                    case 2:
                        curSprite = curRun2;
                        break;
                    case 3:
                        curSprite = curRun3;
                        break;
                }
            }
        }

        public override void HandleSprites()
        {
            switch (state)
            {
                case healthStates.small:

                    curIdle = smallIdle;
                    curRun1 = smallRun1;
                    curRun2 = smallRun2;
                    curRun3 = smallRun3;
                    curJump = smallJump;

                    jumpRange = 0.53f;
                    luigi.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .45f);

                    adult = false;

                    break;

                case healthStates.tall:

                    curIdle = adultIdle;
                    curRun1 = adultRun1;
                    curRun2 = adultRun2;
                    curRun3 = adultRun3;
                    curJump = adultJump;

                    jumpRange = 1f;
                    luigi.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .93f);

                    adult = true;

                    break;

                case healthStates.fire:

                    curIdle = fireIdle;
                    curRun1 = fireRun1;
                    curRun2 = fireRun2;
                    curRun3 = fireRun3;
                    curJump = fireJump;

                    jumpRange = 1f;
                    luigi.GetComponent<CapsuleCollider2D>().size = new Vector2(0.33f, .93f);

                    adult = true;

                    break;

            }
        }

        #endregion

        #region Controllers
        public override void Jump(float jumpForce, bool fallFast, bool playSound)
        {
            if (!canJump) { return; }

            if (fallFast) { rb.gravityScale = 2; }

            if (playSound && !adult)
            {
                GetComponent<PhysicalBehaviour>().PlayClipOnce(clip: jumpSoundSmall, volume: .5f);

            }
            else if (playSound && adult)
            {
                GetComponent<PhysicalBehaviour>().PlayClipOnce(clip: jumpSoundAdult, volume: 1);
            }

            //cant jump again, so you cant infinitely jump in the air
            canJump = false;

            //add force upwards for jump
            rb.AddForce(Vector2.up * jumpForce, ForceMode2D.Impulse);
        }

        public override void Hurt()
        {
            if (invincibilityFrames > 0) { return; }

            if (state == healthStates.small) { Die(); }
            if (state == healthStates.tall || state == healthStates.fire) { state = healthStates.small; HandleSprites(); invincibilityFrames = 1; }
        }

        public override void Die()
        {
            cooldown -= Time.fixedDeltaTime;

            curSprite = smallDeath;

            Jump(150, true, false);

            col.enabled = false;

            if (cooldown <= 0)
            {
                luigi.SetActive(false);
            }
        }

        public override void PowerUp(healthStates powerType)
        {
            state = powerType;
            GetComponent<PhysicalBehaviour>().PlayClipOnce(powerUpSound, 0.5f);
        }

        public override void ShootFire()
        {
            fireSpriteCooldown = 0.15f;

            GameObject fireBall = Instantiate(ModAPI.FindSpawnable("Fire Ball").Prefab, transform.position + new Vector3(0.25f * (transform.localScale.x / transform.localScale.x), 0, 0), Quaternion.identity);
            CatalogBehaviour.PerformMod(ModAPI.FindSpawnable("Fire Ball"), fireBall);

            ModAPI.Notify("fireball shot");

            GetComponent<PhysicalBehaviour>().PlayClipOnce(fireballSound, 1f);

            fireBall.GetComponent<FireBall>().characterThatThrew = luigi;
            fireBall.GetComponent<Rigidbody2D>().AddForce(fireBall.transform.right * transform.localScale.x * 75, ForceMode2D.Impulse);
        }

        #endregion
    }

    public abstract class BasicEnemy : MonoBehaviour
    {
        public Transform enemy;

        public AudioClip squashSound;

        public Rigidbody2D rb;

        public Sprite run1, run2;

        public Sprite squashed;

        public float lRange = 0.6f, rRange = 0.6f, maxSpeed = 2;

        public int walkDir = 1;

        public float deathCooldown = 0.3f;
        public float animCooldown = 0.1f;
        public int animNumb;

        public healthStates state;

        public abstract void Walk();

        public abstract void Animate();

        public abstract void Hurt(float cooldown);

        public abstract void Death();

        public abstract void TestForCollision();
    }

    #region enemies
    public class Goomba : BasicEnemy
    {
        private void FixedUpdate()
        {
            if (state != healthStates.squashed)
            {
                Walk();
            }
        }

        private void Update()
        {
            if (state == healthStates.squashed)
            {
                GetComponent<PhysicalBehaviour>().PlayClipOnce(squashSound, 0.5f);

                deathCooldown -= Time.deltaTime;
                enemy.GetComponent<SpriteRenderer>().sprite = squashed;

                if (deathCooldown <= 0) Death();
            }

            TestForCollision();

            if (state != healthStates.squashed)
            {
                Animate();
            }

            if (GetComponent<PhysicalBehaviour>().IsInLava) { Death(); }
        }

        public override void Walk()
        {
            rb.AddForce(enemy.right * 500 * walkDir);
            rb.velocity = new Vector2(Mathf.Clamp(rb.velocity.x, -maxSpeed, maxSpeed), rb.velocity.y);
        }

        public override void Hurt(float cooldown)
        {
            state = healthStates.squashed;
        }

        public override void Animate()
        {
            animCooldown -= Time.deltaTime;

            if (animCooldown <= 0)
            {
                animNumb++;
                animCooldown = 0.2f;
            }

            if (animNumb > 2)
            {
                animNumb = 1;
            }

            switch (animNumb)
            {
                case 1:
                    GetComponent<SpriteRenderer>().sprite = run1;
                    break;
                case 2:
                    GetComponent<SpriteRenderer>().sprite = run2;
                    break;
            }
        }

        public override void Death()
        {
            gameObject.SetActive(false);
        }

        public override void TestForCollision()
        {
            RaycastHit2D lHit = Physics2D.Raycast(enemy.position, enemy.right, lRange);

            RaycastHit2D rHit = Physics2D.Raycast(enemy.position, -enemy.right, rRange);

            if (lHit)
            {
                transform.localScale = new Vector3(2, 2, 2); walkDir = -1;

                if (lHit.transform.GetComponent<Character>())
                {
                    lHit.transform.GetComponent<Character>().Hurt();
                }
            }

            if (rHit)
            {
                transform.localScale = new Vector3(-2, 2, 2); walkDir = 1;

                if (rHit.transform.GetComponent<Character>())
                {
                    rHit.transform.GetComponent<Character>().Hurt();
                }
            }
        }
    }

    public class Koopa : BasicEnemy
    {
        public float shellTime = 5;

        public bool hitInShell = false;
        public bool red;

        private void FixedUpdate()
        {
            if (state != healthStates.squashed)
            {
                Walk();
                Animate();
            }
        }

        private void Update()
        {
            if (state == healthStates.dead) { return; }

            TestForCollision();

            if (state == healthStates.standard)
            {
                enemy.GetComponent<CapsuleCollider2D>().size = new Vector2(0.43f, .66f);
                maxSpeed = 2;
            }

            if (state == healthStates.squashed)
            {
                Shell();
            }

            if (GetComponent<PhysicalBehaviour>().IsInLava) { Death(); }
        }

        public void Shell()
        {
            shellTime -= Time.deltaTime;

            maxSpeed = 10f;

            GetComponent<SpriteRenderer>().sprite = squashed;
            GetComponent<CapsuleCollider2D>().size = new Vector2(0.43f, 0.35f);

            Walk();

            if (shellTime <= 0) { state = healthStates.standard; }
        }

        public override void Walk()
        {
            rb.AddForce(Vector2.right * 500 * walkDir);
            rb.velocity = new Vector2(Mathf.Clamp(rb.velocity.x, -maxSpeed, maxSpeed), rb.velocity.y);
        }

        public override void Animate()
        {
            if (state != healthStates.squashed)

                animCooldown -= Time.deltaTime;

            if (animCooldown <= 0)
            {
                animNumb++;
                animCooldown = 0.2f;
            }

            if (animNumb > 2)
            {
                animNumb = 1;
            }

            switch (animNumb)
            {
                case 1:
                    GetComponent<SpriteRenderer>().sprite = run1;
                    break;
                case 2:
                    GetComponent<SpriteRenderer>().sprite = run2;
                    break;
            }
        }

        public override void Death()
        {
            state = healthStates.dead;

            transform.localScale = new Vector3(2, 2, 2);
            GetComponent<SpriteRenderer>().sprite = squashed;

            rb.freezeRotation = false;
            rb.AddForce(Vector2.up * 75, ForceMode2D.Impulse);
            GetComponent<CapsuleCollider2D>().enabled = false;
        }

        public override void Hurt(float cooldown)
        {
            state = healthStates.squashed;

            shellTime = cooldown;
        }

        public override void TestForCollision()
        {

            RaycastHit2D lHit = Physics2D.Raycast(enemy.position, enemy.right, lRange);
            RaycastHit2D rHit = Physics2D.Raycast(enemy.position, -enemy.right, rRange);

            //if something is to the left flip to right
            if (lHit)
            {
                transform.localScale = new Vector3(-2, 2, 2); walkDir = -1;

                if (lHit.transform.GetComponent<Character>())
                {
                    lHit.transform.GetComponent<Character>().Hurt();
                }
            }

            //opposite
            if (rHit)
            {
                transform.localScale = new Vector3(2, 2, 2); walkDir = 1;

                if (rHit.transform.GetComponent<Character>())
                {
                    rHit.transform.GetComponent<Character>().Hurt();
                }
            }
        }
    }

    #endregion

    public class StandardBlock : MonoBehaviour
    {
        public GameObject notPlaced;
        public GameObject block;

        public AudioClip breakSound;

        public GameObject blockHolder;

        public int coinAmount;
        public int animNumber;

        public string itemName = "Mushroom";

        private float animCooldown;

        public bool canBeBroken;
        public bool animated;

        public bool canHoldItems;
        public bool canBeOpened;
        public bool spitOut;

        public bool followsMouse = true;

        public string material;

        Vector3 curSpot;
        Vector3 lastSpot;

        Item HeldItem;

        bool placed;

        public Sprite block1, block2, block3;
        
        private void Update()
        {
            if (spitOut) { transform.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Blocks/emptyBlock.png"); }

            if (!placed && followsMouse)
            {
                canBeOpened = false;

                if (!Input.GetKey(KeyCode.LeftAlt))
                    curSpot = new Vector2(Mathf.RoundToInt(Global.main.MousePosition.x), Mathf.RoundToInt(Global.main.MousePosition.y));
                else
                    curSpot = Global.main.MousePosition;

                if (Input.GetMouseButtonDown(0)) { ModAPI.Notify("Q and E to place blocks, alt to free move block"); block.SetActive(false); }

                block.transform.position = Vector2.Lerp(block.transform.position, curSpot, Time.deltaTime * 44f);

                if (Input.GetKey(KeyCode.E) || Input.GetKey("Q")) { block.transform.position = curSpot; }

                if (Input.GetKey(KeyCode.Q) && curSpot != lastSpot || Input.GetKey(KeyCode.E) && curSpot != lastSpot) { PlaceBlock(); }
            }

        }

        void PlaceBlock()
        {
            placed = true;
            canBeOpened = true;

            lastSpot = curSpot;

            block.isStatic = true;
            block.GetComponent<BoxCollider2D>().enabled = true;
            block.GetComponent<Rigidbody2D>().bodyType = RigidbodyType2D.Kinematic;

            this.transform.parent = blockHolder.transform;
            UnityEngine.GameObject.Destroy(block.GetComponent<GlowingHotMetalBehaviour>());
        }

        public void Break()
        {
            if (canHoldItems && !spitOut)
            {
                SpitOutPowerUp(itemName);
            }

            if (coinAmount <= 0)
            {
                if (!canBeBroken) { return; }

                GetComponent<PhysicalBehaviour>().PlayClipOnce(breakSound, 0.5f);

                gameObject.SetActive(false);
            }
        }

        void SpitOutPowerUp(string PowerUpName)
        {
            if(!canBeOpened) { return; }

            spitOut = true;

            GameObject powerUp = Instantiate(ModAPI.FindSpawnable(PowerUpName).Prefab,
                new Vector3(transform.position.x, transform.position.y + 1), Quaternion.identity);

            
            CatalogBehaviour.PerformMod(ModAPI.FindSpawnable(PowerUpName), powerUp);

            GameObject emptyBlock = Instantiate(ModAPI.FindSpawnable("Empty Block").Prefab, transform.position, Quaternion.identity);
            CatalogBehaviour.PerformMod(ModAPI.FindSpawnable("Empty Block"), emptyBlock);

            emptyBlock.GetComponent<StandardBlock>().followsMouse = false;
            emptyBlock.GetComponent<StandardBlock>().placed = true;
            emptyBlock.transform.position = transform.position;

            gameObject.SetActive(false);
        }
    }

    public class SquareDraw : MonoBehaviour
    {
        private Vector3 startPos;
        private Vector3 endPos;

        private bool placed;

        public  GameObject square;

        public Sprite backgroundSprite;

        private void Update()
        {
            if (!placed)
            {
                if (Input.GetMouseButtonDown(0))
                {
                    startPos = new Vector3(Mathf.RoundToInt(Global.main.MousePosition.x), Mathf.RoundToInt(Global.main.MousePosition.y), 0);
                }

                if (Input.GetMouseButton(0))
                {
                    endPos = new Vector3(Mathf.RoundToInt(Global.main.MousePosition.x), Mathf.RoundToInt(Global.main.MousePosition.y), 0);

                    float width = Mathf.Abs(startPos.x - endPos.x);
                    float height = Mathf.Abs(startPos.y - endPos.y);

                    square.transform.localScale = new Vector3(width, height, 1);
                    square.transform.position = new Vector3((startPos.x + endPos.x) / 2, (startPos.y + endPos.y) / 2, 0);
                }
            }
            
            if (Input.GetMouseButtonUp(0))
            {
                placed = true;

                GetComponent<SpriteRenderer>().sortingOrder = -5;
            }
        }
    }

    #region itemStuff
    public abstract class Item : MonoBehaviour
    {
        public Sprite[] itemSprites;

        public Rigidbody2D rb;
        public int moveDir;
        public abstract void OnPickup(Character characterTouched);
        public abstract void Move();
    }

    public class Mushroom : Item
    {
        bool right;

        public void FixedUpdate()
        {
            Move();
        }

        public override void Move()
        {
            GetComponent<Rigidbody2D>().AddForce(transform.right * 500);
            GetComponent<Rigidbody2D>().velocity = new Vector2(Mathf.Clamp(GetComponent<Rigidbody2D>().velocity.x, -2, 2), GetComponent<Rigidbody2D>().velocity.y);
        }

        public override void OnPickup(Character characterTouched)
        {
            if(characterTouched.state != healthStates.fire)
                characterTouched.PowerUp(healthStates.tall);

            gameObject.SetActive(false);
        }

        private void OnCollisionEnter2D(Collision2D col)
        {
            if(col.transform.GetComponent<Character>())
            {
                OnPickup(col.transform.GetComponent<Character>());
            }
        }
    }

    public class FireFlower : Item 
    {
        public override void Move()
        {
            //doesnt move
        }

        public override void OnPickup(Character characterTouched)
        {
            characterTouched.PowerUp(healthStates.fire);

            gameObject.SetActive(false);
        }

        private void OnCollisionEnter2D(Collision2D col)
        {
            if (col.transform.GetComponent<Character>())
            {
                OnPickup(col.transform.GetComponent<Character>());
            }
        }
    }

    public class FireBall : MonoBehaviour 
    {
        public Rigidbody2D rb;
        public CircleCollider2D col;

        public int bounceAmount;

        public GameObject characterThatThrew;

        public void FixedUpdate()
        {
            Physics2D.IgnoreCollision(col, characterThatThrew.GetComponent<CapsuleCollider2D>());

            rb.gravityScale = 2;

            if(bounceAmount > 4)
            {
                gameObject.SetActive(false);

            }
        }

        void OnCollisionEnter2D(Collision2D coll)
        {
            if (!coll.transform.GetComponent<BasicEnemy>())
            {
                rb.AddForce(Vector2.up * 125, ForceMode2D.Impulse);
                bounceAmount++;
            }
            else
            {
                if(coll.transform.GetComponent<Goomba>()) { coll.transform.GetComponent<BasicEnemy>().Hurt(0); }
                if(coll.transform.GetComponent<Koopa>()) { coll.transform.GetComponent<BasicEnemy>().Death(); }

                gameObject.SetActive(false);
            }
            
        }
    }

    #endregion

}